/* --------------------------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
const code = require('vscode');
const ls = require('vscode-languageserver-types');
const is = require('./utils/is');
const protocolCompletionItem_1 = require('./protocolCompletionItem');
const protocolCodeLens_1 = require('./protocolCodeLens');
function createConverter(uriConverter) {
    const nullConverter = (value) => code.Uri.parse(value);
    const _uriConverter = uriConverter || nullConverter;
    function asUri(value) {
        return _uriConverter(value);
    }
    function asDiagnostics(diagnostics) {
        return diagnostics.map(asDiagnostic);
    }
    function asDiagnostic(diagnostic) {
        let result = new code.Diagnostic(asRange(diagnostic.range), diagnostic.message, asDiagnosticSeverity(diagnostic.severity));
        if (is.defined(diagnostic.code)) {
            result.code = diagnostic.code;
        }
        if (is.defined(diagnostic.source)) {
            result.source = diagnostic.source;
        }
        return result;
    }
    function asRange(value) {
        if (is.undefined(value)) {
            return undefined;
        }
        else if (is.nil(value)) {
            return null;
        }
        return new code.Range(asPosition(value.start), asPosition(value.end));
    }
    function asPosition(value) {
        if (is.undefined(value)) {
            return undefined;
        }
        else if (is.nil(value)) {
            return null;
        }
        return new code.Position(value.line, value.character);
    }
    function asDiagnosticSeverity(value) {
        if (is.undefined(value) || is.nil(value)) {
            return code.DiagnosticSeverity.Error;
        }
        switch (value) {
            case 1 /* Error */:
                return code.DiagnosticSeverity.Error;
            case 2 /* Warning */:
                return code.DiagnosticSeverity.Warning;
            case 3 /* Information */:
                return code.DiagnosticSeverity.Information;
            case 4 /* Hint */:
                return code.DiagnosticSeverity.Hint;
        }
        return code.DiagnosticSeverity.Error;
    }
    function asHover(hover) {
        if (is.undefined(hover)) {
            return undefined;
        }
        if (is.nil(hover)) {
            return null;
        }
        return new code.Hover(hover.contents, is.defined(hover.range) ? asRange(hover.range) : undefined);
    }
    function asCompletionResult(result) {
        if (is.undefined(result)) {
            return undefined;
        }
        else if (is.nil(result)) {
            return null;
        }
        if (Array.isArray(result)) {
            let items = result;
            return items.map(asCompletionItem);
        }
        let list = result;
        return new code.CompletionList(list.items.map(asCompletionItem), list.isIncomplete);
    }
    function set(value, func) {
        if (is.defined(value)) {
            func();
        }
    }
    function asCompletionItem(item) {
        let result = new protocolCompletionItem_1.default(item.label);
        set(item.detail, () => result.detail = item.detail);
        set(item.documentation, () => result.documentation = item.documentation);
        set(item.filterText, () => result.filterText = item.filterText);
        set(item.insertText, () => result.insertText = asCompletionInsertText(item.insertText));
        set(item.range, () => result.range = asRange(item.range));
        // Protocol item kind is 1 based, codes item kind is zero based.
        set(item.kind, () => result.kind = item.kind - 1);
        set(item.sortText, () => result.sortText = item.sortText);
        set(item.textEdit, () => result.textEdit = asTextEdit(item.textEdit));
        set(item.additionalTextEdits, () => result.additionalTextEdits = asTextEdits(item.additionalTextEdits));
        set(item.command, () => result.command = asCommand(item.command));
        set(item.data, () => result.data = item.data);
        return result;
    }
    function asCompletionInsertText(text) {
        if (is.string(text)) {
            return text;
        }
        else if (ls.SnippetString.is(text)) {
            return new code.SnippetString(text.value);
        }
        return undefined;
    }
    function asTextEdit(edit) {
        return new code.TextEdit(asRange(edit.range), edit.newText);
    }
    function asTextEdits(items) {
        if (is.undefined(items)) {
            return undefined;
        }
        else if (is.nil(items)) {
            return null;
        }
        return items.map(asTextEdit);
    }
    function asSignatureHelp(item) {
        if (is.undefined(item)) {
            return undefined;
        }
        else if (is.nil(item)) {
            return null;
        }
        let result = new code.SignatureHelp();
        set(item.activeParameter, () => result.activeParameter = item.activeParameter);
        set(item.activeSignature, () => result.activeSignature = item.activeSignature);
        set(item.signatures, () => result.signatures = asSignatureInformations(item.signatures));
        return result;
    }
    function asSignatureInformations(items) {
        return items.map(asSignatureInformation);
    }
    function asSignatureInformation(item) {
        let result = new code.SignatureInformation(item.label);
        set(item.documentation, () => result.documentation = item.documentation);
        set(item.parameters, () => result.parameters = asParameterInformations(item.parameters));
        return result;
    }
    function asParameterInformations(item) {
        return item.map(asParameterInformation);
    }
    function asParameterInformation(item) {
        let result = new code.ParameterInformation(item.label);
        set(item.documentation, () => result.documentation = item.documentation);
        return result;
    }
    function asDefinitionResult(item) {
        if (is.undefined(item)) {
            return undefined;
        }
        else if (is.nil(item)) {
            return null;
        }
        if (is.array(item)) {
            return item.map(asLocation);
        }
        else {
            return asLocation(item);
        }
    }
    function asLocation(item) {
        if (is.undefined(item)) {
            return undefined;
        }
        if (is.nil(item)) {
            return null;
        }
        return new code.Location(_uriConverter(item.uri), asRange(item.range));
    }
    function asReferences(values) {
        if (is.undefined(values)) {
            return undefined;
        }
        if (is.nil(values)) {
            return null;
        }
        return values.map(asLocation);
    }
    function asDocumentHighlights(values) {
        if (is.undefined(values)) {
            return undefined;
        }
        if (is.nil(values)) {
            return null;
        }
        return values.map(asDocumentHighlight);
    }
    function asDocumentHighlight(item) {
        let result = new code.DocumentHighlight(asRange(item.range));
        set(item.kind, () => result.kind = asDocumentHighlightKind(item.kind));
        return result;
    }
    function asDocumentHighlightKind(item) {
        switch (item) {
            case 1 /* Text */:
                return code.DocumentHighlightKind.Text;
            case 2 /* Read */:
                return code.DocumentHighlightKind.Read;
            case 3 /* Write */:
                return code.DocumentHighlightKind.Write;
        }
        return code.DocumentHighlightKind.Text;
    }
    function asSymbolInformations(values, uri) {
        if (is.undefined(values)) {
            return undefined;
        }
        if (is.nil(values)) {
            return null;
        }
        return values.map(information => asSymbolInformation(information, uri));
    }
    function asSymbolInformation(item, uri) {
        // Symbol kind is one based in the protocol and zero based in code.
        let result = new code.SymbolInformation(item.name, item.kind - 1, asRange(item.location.range), item.location.uri ? _uriConverter(item.location.uri) : uri);
        set(item.containerName, () => result.containerName = item.containerName);
        return result;
    }
    function asCommand(item) {
        let result = { title: item.title, command: item.command };
        set(item.arguments, () => result.arguments = item.arguments);
        return result;
    }
    function asCommands(items) {
        if (is.undefined(items)) {
            return undefined;
        }
        if (is.nil(items)) {
            return null;
        }
        return items.map(asCommand);
    }
    function asCodeLens(item) {
        let result = new protocolCodeLens_1.default(asRange(item.range));
        if (is.defined(item.command))
            result.command = asCommand(item.command);
        if (is.defined(item.data))
            result.data = item.data;
        return result;
    }
    function asCodeLenses(items) {
        if (is.undefined(items)) {
            return undefined;
        }
        if (is.nil(items)) {
            return null;
        }
        return items.map(asCodeLens);
    }
    function asWorkspaceEdit(item) {
        if (is.undefined(item)) {
            return undefined;
        }
        if (is.nil(item)) {
            return null;
        }
        let result = new code.WorkspaceEdit();
        let keys = Object.keys(item.changes);
        keys.forEach(key => result.set(_uriConverter(key), asTextEdits(item.changes[key])));
        return result;
    }
    function asDocumentLink(item) {
        let range = asRange(item.range);
        let target = is.defined(item.target) && asUri(item.target);
        return new code.DocumentLink(range, target);
    }
    function asDocumentLinks(items) {
        if (is.undefined(items)) {
            return undefined;
        }
        if (is.nil(items)) {
            return null;
        }
        return items.map(asDocumentLink);
    }
    return {
        asUri,
        asDiagnostics,
        asDiagnostic,
        asRange,
        asPosition,
        asDiagnosticSeverity,
        asHover,
        asCompletionResult,
        asCompletionItem,
        asTextEdit,
        asTextEdits,
        asSignatureHelp,
        asSignatureInformations,
        asSignatureInformation,
        asParameterInformations,
        asParameterInformation,
        asDefinitionResult,
        asLocation,
        asReferences,
        asDocumentHighlights,
        asDocumentHighlight,
        asDocumentHighlightKind,
        asSymbolInformations,
        asSymbolInformation,
        asCommand,
        asCommands,
        asCodeLens,
        asCodeLenses,
        asWorkspaceEdit,
        asDocumentLink,
        asDocumentLinks
    };
}
exports.createConverter = createConverter;
