/// <reference types="node" />
/**
 * A language server message
 */
export interface Message {
    jsonrpc: string;
}
/**
 * Request message
 */
export interface RequestMessage extends Message {
    /**
     * The request id.
     */
    id: number | string;
    /**
     * The method to be invoked.
     */
    method: string;
    /**
     * The method's params.
     */
    params?: any;
}
/**
 * Predefined error codes.
 */
export declare namespace ErrorCodes {
    const ParseError: number;
    const InvalidRequest: number;
    const MethodNotFound: number;
    const InvalidParams: number;
    const InternalError: number;
    const serverErrorStart: number;
    const serverErrorEnd: number;
    const MessageWriteError: number;
    const MessageReadError: number;
}
export interface ResponseErrorLiteral<D> {
    /**
     * A number indicating the error type that occured.
     */
    code: number;
    /**
     * A string providing a short decription of the error.
     */
    message: string;
    /**
     * A Primitive or Structured value that contains additional
     * information about the error. Can be omitted.
     */
    data?: D;
}
/**
 * A error object return in a response in case a request
 * has failed.
 */
export declare class ResponseError<D> extends Error {
    code: number;
    message: string;
    data: D;
    constructor(code: number, message: string, data?: D);
    toJson(): ResponseErrorLiteral<D>;
}
/**
 * A response message.
 */
export interface ResponseMessage extends Message {
    /**
     * The request id.
     */
    id: number | string;
    /**
     * The result of a request. This can be omitted in
     * the case of an error.
     */
    result?: any;
    /**
     * The error object in case a request fails.
     */
    error?: ResponseErrorLiteral<any>;
}
/**
 * An interface to type messages.
 */
export interface MessageType {
    method: string;
}
/**
 * End marker interface for request and notification types.
 */
export interface _EM {
    _$endMarker$_: number;
}
/**
 * A interface to type the request parameter / response pair
 */
export interface RequestType0<R, E, RO> extends MessageType {
    _: [R, E, RO, _EM];
    registerOptions?: RO;
}
export interface RequestType<P, R, E, RO> extends MessageType {
    _: [P, R, E, RO, _EM];
    registerOptions?: RO;
}
export interface RequestType1<P1, R, E, RO> extends MessageType {
    _: [P1, R, E, RO, _EM];
    registerOptions?: RO;
}
export interface RequestType2<P1, P2, R, E, RO> extends MessageType {
    _: [P1, P2, R, E, RO, _EM];
    registerOptions?: RO;
}
export interface RequestType3<P1, P2, P3, R, E, RO> extends MessageType {
    _: [P1, P2, P3, R, E, RO, _EM];
    registerOptions?: RO;
}
export interface RequestType4<P1, P2, P3, P4, R, E, RO> extends MessageType {
    _: [P1, P2, P3, P4, R, E, RO, _EM];
    registerOptions?: RO;
}
export interface RequestType5<P1, P2, P3, P4, P5, R, E, RO> extends MessageType {
    _: [P1, P2, P3, P4, P5, R, E, RO, _EM];
    registerOptions?: RO;
}
export interface RequestType6<P1, P2, P3, P4, P5, P6, R, E, RO> extends MessageType {
    _: [P1, P2, P3, P4, P5, P6, R, E, RO, _EM];
    registerOptions?: RO;
}
export interface RequestType7<P1, P2, P3, P4, P5, P6, P7, R, E, RO> extends MessageType {
    _: [P1, P2, P3, P4, P5, P6, P7, R, E, RO, _EM];
    registerOptions?: RO;
}
export interface RequestType8<P1, P2, P3, P4, P5, P6, P7, P8, R, E, RO> extends MessageType {
    _: [P1, P2, P3, P4, P5, P6, P7, P8, R, E, RO, _EM];
    registerOptions?: RO;
}
export interface RequestType9<P1, P2, P3, P4, P5, P6, P7, P8, P9, R, E, RO> extends MessageType {
    _: [P1, P2, P3, P4, P5, P6, P7, P8, P9, R, E, RO, _EM];
    registerOptions?: RO;
}
/**
 * Notification Message
 */
export interface NotificationMessage extends Message {
    /**
     * The method to be invoked.
     */
    method: string;
    /**
     * The notification's params.
     */
    params?: any;
}
/**
 * An interface to type notification messages.
 */
export interface NotificationType<P, RO> extends MessageType {
    _: [P, RO, _EM];
    registerOptions?: RO;
}
export interface NotificationType0<RO> extends MessageType {
    _: [RO, _EM];
    registerOptions?: RO;
}
export interface NotificationType1<P1, RO> extends MessageType {
    _: [P1, RO, _EM];
    registerOptions?: RO;
}
export interface NotificationType2<P1, P2, RO> extends MessageType {
    _: [P1, P2, RO, _EM];
    registerOptions?: RO;
}
export interface NotificationType3<P1, P2, P3, RO> extends MessageType {
    _: [P1, P2, P3, RO, _EM];
    registerOptions?: RO;
}
export interface NotificationType4<P1, P2, P3, P4, RO> extends MessageType {
    _: [P1, P2, P3, P4, RO, _EM];
    registerOptions?: RO;
}
export interface NotificationType5<P1, P2, P3, P4, P5, RO> extends MessageType {
    _: [P1, P2, P3, P4, P5, RO, _EM];
    registerOptions?: RO;
}
export interface NotificationType6<P1, P2, P3, P4, P5, P6, RO> extends MessageType {
    _: [P1, P2, P3, P4, P5, P6, RO, _EM];
    registerOptions?: RO;
}
export interface NotificationType7<P1, P2, P3, P4, P5, P6, P7, RO> extends MessageType {
    _: [P1, P2, P3, P4, P5, P6, P7, RO, _EM];
    registerOptions?: RO;
}
export interface NotificationType8<P1, P2, P3, P4, P5, P6, P7, P8, RO> extends MessageType {
    _: [P1, P2, P3, P4, P5, P6, P7, P8, RO, _EM];
    registerOptions?: RO;
}
export interface NotificationType9<P1, P2, P3, P4, P5, P6, P7, P8, P9, RO> extends MessageType {
    _: [P1, P2, P3, P4, P5, P6, P7, P8, P9, RO, _EM];
    registerOptions?: RO;
}
/**
 * Tests if the given message is a request message
 */
export declare function isRequestMessage(message: Message): message is RequestMessage;
/**
 * Tests if the given message is a notification message
 */
export declare function isNotificationMessage(message: Message): message is NotificationMessage;
/**
 * Tests if the given message is a response message
 */
export declare function isReponseMessage(message: Message): message is ResponseMessage;
